package org.owasp.lmHash;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;

/**
 * From: http://forums.sun.com/thread.jspa?threadID=742735
 * http://davenport.sourceforge.net/ntlm.html#lmHash Output checked at:
 * http://www.lmcrack.com/
 */
public class LMHasher {

	public byte[] digest(String stringToHash) {
		byte[] oemPassword;
		try {
			oemPassword = stringToHash.toUpperCase().getBytes("US-ASCII");
			int length = Math.min(oemPassword.length, 14);
			byte[] keyBytes = new byte[14];
			System.arraycopy(oemPassword, 0, keyBytes, 0, length);
			Key lowKey = createDESKey(keyBytes, 0);
			Key highKey = createDESKey(keyBytes, 7);
			byte[] magicConstant = "KGS!@#$%".getBytes("US-ASCII");
			Cipher des = Cipher.getInstance("DES/ECB/NoPadding");
			des.init(Cipher.ENCRYPT_MODE, lowKey);
			byte[] lowHash = des.doFinal(magicConstant);
			des.init(Cipher.ENCRYPT_MODE, highKey);
			byte[] highHash = des.doFinal(magicConstant);
			byte[] lmHash = new byte[16];
			System.arraycopy(lowHash, 0, lmHash, 0, 8);
			System.arraycopy(highHash, 0, lmHash, 8, 8);
			return lmHash;
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (NoSuchPaddingException e) {
			e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			e.printStackTrace();
		} catch (BadPaddingException e) {
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Creates a DES encryption key from the given key material.
	 * 
	 * @param bytes
	 *            A byte array containing the DES key material.
	 * @param offset
	 *            The offset in the given byte array at which the 7-byte key
	 *            material starts.
	 * 
	 * @return A DES encryption key created from the key material starting at
	 *         the specified offset in the given byte array.
	 */
	private Key createDESKey(byte[] bytes, int offset) {
		byte[] keyBytes = new byte[7];
		System.arraycopy(bytes, offset, keyBytes, 0, 7);
		byte[] material = new byte[8];
		material[0] = keyBytes[0];
		material[1] = (byte) (keyBytes[0] << 7 | (keyBytes[1] & 0xff) >>> 1);
		material[2] = (byte) (keyBytes[1] << 6 | (keyBytes[2] & 0xff) >>> 2);
		material[3] = (byte) (keyBytes[2] << 5 | (keyBytes[3] & 0xff) >>> 3);
		material[4] = (byte) (keyBytes[3] << 4 | (keyBytes[4] & 0xff) >>> 4);
		material[5] = (byte) (keyBytes[4] << 3 | (keyBytes[5] & 0xff) >>> 5);
		material[6] = (byte) (keyBytes[5] << 2 | (keyBytes[6] & 0xff) >>> 6);
		material[7] = (byte) (keyBytes[6] << 1);
		oddParity(material);
		return new SecretKeySpec(material, "DES");
	}

	/**
	 * Applies odd parity to the given byte array.
	 * 
	 * @param bytes
	 *            The data whose parity bits are to be adjusted for odd parity.
	 */
	private void oddParity(byte[] bytes) {
		for (int i = 0; i < bytes.length; i++) {
			byte b = bytes[i];
			boolean needsParity = (((b >>> 7) ^ (b >>> 6) ^ (b >>> 5)
					^ (b >>> 4) ^ (b >>> 3) ^ (b >>> 2) ^ (b >>> 1)) & 0x01) == 0;
			if (needsParity) {
				bytes[i] |= (byte) 0x01;
			} else {
				bytes[i] &= (byte) 0xfe;
			}
		}
	}
}